// 3D Vector Class by Arsham Skrenes (c)2005
// CPSC 314 - University of British Columbia

#include "3dvector.h"

C3dVector::C3dVector(void)
{
	x = y = z = 0;
	auto_normalize = false;
}
C3dVector::C3dVector(double x, double y, double z)
{
	(*this).x = x;
	(*this).y = y;
	(*this).z = z;
	(*this).auto_normalize = false;
}
C3dVector::C3dVector(double x, double y, double z, bool auto_normalize)
{
	(*this).x = x;
	(*this).y = y;
	(*this).z = z;
	(*this).auto_normalize = auto_normalize;
	if(auto_normalize) normalize();
}
C3dVector::C3dVector(const C3dVector &other)
{
	x = other.x;
	y = other.y;
	z = other.z;
	auto_normalize = false;
}
C3dVector::~C3dVector(void)
{
}

void C3dVector::setXYZ(double x_value, double y_value, double z_value)
{
	x = x_value;
	y = y_value;
	z = z_value;
	if(auto_normalize) normalize();
}
void C3dVector::setX(double x_value)
{
	x = x_value;
}
void C3dVector::setY(double y_value)
{
	y = y_value;
}
void C3dVector::setZ(double z_value)
{
	z = z_value;
}
double C3dVector::getX(void)
{
	return x;
}
double C3dVector::getY(void)
{
	return y;
}
double C3dVector::getZ(void)
{
	return z;
}
double C3dVector::getLength(void)
{
	return sqrt	(dotProduct(*this));
}

void C3dVector::normalize(void)
{
	double length = getLength();
	if (length == 0) return; // prevent divide by zero
	x = x/length;
	y = y/length;
	z = z/length;
}
void C3dVector::autoNormalize(bool set)
{
	auto_normalize = set;
	if(auto_normalize) normalize();
}

void C3dVector::operator= (const C3dVector other)
{
	x = other.x;
	y = other.y;
	z = other.z;

	if(auto_normalize) normalize();
}
bool C3dVector::operator== ( const C3dVector other)
{
	return x==other.x && y==other.y && z==other.z;
}
bool C3dVector::operator!= ( const C3dVector other)
{
	return x!=other.x || y!=other.y || z!=other.z;
}
void C3dVector::operator+= (const C3dVector other)
{
	x += other.x;
	y += other.y;
	z += other.z;
	if(auto_normalize) normalize();
}
void C3dVector::operator-= (const C3dVector other)
{
	x -= other.x;
	y -= other.y;
	z -= other.z;
	if(auto_normalize) normalize();
}
C3dVector C3dVector::operator+ (const C3dVector other)
{
	return C3dVector(x + other.x, y + other.y, z + other.z);
}
C3dVector C3dVector::operator- (const C3dVector other)
{
	return C3dVector(x - other.x, y - other.y, z - other.z);
}
C3dVector C3dVector::operator* (const C3dVector other)
{
	return C3dVector(	y*other.z - z*other.y,
						z*other.x - x*other.z,
						x*other.y - y*other.x);
}
C3dVector C3dVector::operator* (double scalar)
{
	return C3dVector(scalar*x, scalar*y, scalar*z);
}
double C3dVector::dotProduct(const C3dVector other)
{
	return	x*other.x + y*other.y + z*other.z;
}
C3dVector C3dVector::component_wise_multiply(const C3dVector other)
{
	return C3dVector(x * other.x, y * other.y, z * other.z);
}

std::ostream& operator<<(std::ostream& pStr, const C3dVector& other)
{
	return (pStr << "<" << other.x << ", " << other.y << ", " << other.z << ">");
}
